@@ -7,6 +7,7 @@ use atty::Stream;
 use clap::{crate_version, App, AppSettings, Arg};
 use std::{env, ffi::OsStr, path::Path};
 use typed_arena::Arena;
+use walkdir::WalkDir;
 
 use difftastic::*;
 use difftastic::{
@@ -147,7 +148,6 @@ fn main() {
 
     reset_sigpipe();
     configure_color();
-    let arena = Arena::new();
 
     let mode = parse_args();
     let (display_path, lhs_path, rhs_path) = match mode {
@@ -158,7 +158,7 @@ fn main() {
                 Some(ts_lang) => {
                     let bytes = read_or_die(&path);
                     let src = String::from_utf8_lossy(&bytes).to_string();
-                    let tree = tsp::parse_to_tree(&src, &ts_lang);
+                    let (tree, _) = tsp::parse_to_tree(&src, &ts_lang);
                     tsp::print_tree(&tree);
                 }
                 None => {
@@ -174,6 +174,7 @@ fn main() {
                 Some(ts_lang) => {
                     let bytes = read_or_die(&path);
                     let src = String::from_utf8_lossy(&bytes).to_string();
+                    let arena = Arena::new();
                     let ast = tsp::parse(&arena, &src, &ts_lang);
                     init_info(&ast, &[]);
                     println!("{:#?}", ast);
@@ -191,13 +192,22 @@ fn main() {
         } => (display_path, lhs_path, rhs_path),
     };
 
-    let lhs_bytes = read_or_die(&lhs_path);
-    let rhs_bytes = read_or_die(&rhs_path);
+    if Path::new(&lhs_path).is_dir() && Path::new(&rhs_path).is_dir() {
+        diff_directories(&lhs_path, &rhs_path);
+    } else {
+        diff_file(&display_path, &lhs_path, &rhs_path);
+    }
+}
+
+// TODO: prefer PathBuf to &str for paths.
+fn diff_file(display_path: &str, lhs_path: &str, rhs_path: &str) {
+    let lhs_bytes = read_or_die(lhs_path);
+    let rhs_bytes = read_or_die(rhs_path);
 
     let lhs_binary = is_probably_binary(&lhs_bytes);
     let rhs_binary = is_probably_binary(&rhs_bytes);
     if lhs_binary || rhs_binary {
-        println!("{}", style::header(&display_path, "binary"));
+        println!("{}", style::header(display_path, "binary"));
         return;
     }
 
@@ -213,6 +223,7 @@ fn main() {
     let extension = extension.unwrap_or_else(|| OsStr::new(""));
     let ts_lang = tsp::from_extension(extension);
 
+    let arena = Arena::new();
     let (lang_name, lhs, rhs) = match ts_lang {
         Some(ts_lang) => (
             ts_lang.name,
@@ -226,7 +237,7 @@ fn main() {
         ),
     };
 
-    println!("{}", style::header(&display_path, lang_name));
+    println!("{}", style::header(display_path, lang_name));
 
     init_info(&lhs, &rhs);
     mark_syntax(lhs.get(0).copied(), rhs.get(0).copied());
@@ -271,3 +282,26 @@ fn main() {
     }
     println!();
 }
+
+/// Given two directories that contain the files, compare them
+/// pairwise.
+///
+/// This is how the hg extdiff extension calls a diff tool when there
+/// is more than one changed file.
+fn diff_directories(lhs_dir: &str, rhs_dir: &str) {
+    for entry in WalkDir::new(lhs_dir).into_iter().filter_map(Result::ok) {
+        let lhs_path = entry.path();
+        if lhs_path.is_dir() {
+            continue;
+        }
+
+        let rel_path = lhs_path.strip_prefix(lhs_dir).unwrap();
+        let rhs_path = Path::new(rhs_dir).join(rel_path);
+
+        diff_file(
+            &rel_path.to_string_lossy(),
+            &lhs_path.to_string_lossy(),
+            &rhs_path.to_string_lossy(),
+        );
+    }
+}
