@@ -1,23 +1,29 @@
+//! Side-by-side (two column) display of diffs.
+
 use atty::Stream;
 use colored::{Color, Colorize};
-use std::cmp::{max, min};
-use std::collections::HashMap;
+use std::{
+    cmp::{max, min},
+    collections::HashMap,
+};
 
-use crate::lines::{
-    codepoint_len, enforce_exact_length, enforce_max_length, format_line_num, LineGroup, LineNumber,
+use crate::{
+    lines::{
+        codepoint_len, enforce_exact_length, enforce_max_length, format_line_num, LineGroup,
+        LineNumber,
+    },
+    style::apply_colors,
+    syntax::{aligned_lines, MatchedPos},
 };
-use crate::style::apply_colors;
-use crate::syntax::{aligned_lines, MatchedPos};
 
 const SPACER: &str = "  ";
-const MIN_WIDTH: usize = 35;
 
 fn term_width() -> Option<usize> {
     term_size::dimensions().map(|(w, _)| w)
 }
 
 /// Split `s` by newlines, but guarantees that the output is nonempty.
-/// An empty string is consider a single line of length zero.
+/// An empty string is considered as a single line of length zero.
 fn split_lines_nonempty(s: &str) -> Vec<String> {
     if s == "" {
         return vec!["".into()];
@@ -30,22 +36,6 @@ fn split_lines_nonempty(s: &str) -> Vec<String> {
     lines
 }
 
-fn longest_visible_line_lhs(s: &str, groups: &[LineGroup]) -> usize {
-    let lines = split_lines_nonempty(s);
-    let mut longest = 0;
-
-    for group in groups {
-        if let Some(lhs_lines) = &group.lhs_lines {
-            for line_num in lhs_lines.start.0..lhs_lines.end.0 {
-                let current_len = codepoint_len(&lines[line_num]);
-                longest = max(longest, current_len);
-            }
-        }
-    }
-
-    longest
-}
-
 fn longest_visible_line_rhs(s: &str, groups: &[LineGroup]) -> usize {
     let lines = split_lines_nonempty(s);
     let mut longest = 1;
@@ -62,17 +52,8 @@ fn longest_visible_line_rhs(s: &str, groups: &[LineGroup]) -> usize {
     longest
 }
 
-fn lhs_printable_width(
-    lhs: &str,
-    groups: &[LineGroup],
-    lhs_column_width: usize,
-    terminal_width: usize,
-) -> usize {
-    let longest_line_length = longest_visible_line_lhs(lhs, groups);
-    let longest_line = longest_line_length + lhs_column_width;
-
-    let space_available = (terminal_width - SPACER.len()) / 2;
-    max(MIN_WIDTH, min(longest_line, space_available))
+fn lhs_printable_width(terminal_width: usize) -> usize {
+    (terminal_width - SPACER.len()) / 2
 }
 
 fn rhs_printable_width(
@@ -87,7 +68,7 @@ fn rhs_printable_width(
 
     let space_available = (terminal_width - SPACER.len()) - lhs_width;
 
-    max(MIN_WIDTH, min(longest_line, space_available))
+    min(longest_line, space_available)
 }
 
 fn format_line_num_padded(line_num: LineNumber, column_width: usize) -> String {
@@ -258,8 +239,7 @@ pub fn display(
 
     let terminal_width = term_width().unwrap_or(80);
 
-    let lhs_formatted_length =
-        lhs_printable_width(lhs_src, groups, lhs_column_width, terminal_width);
+    let lhs_formatted_length = lhs_printable_width(terminal_width);
     let rhs_formatted_length = rhs_printable_width(
         rhs_src,
         groups,
