@@ -3,7 +3,7 @@
 use crate::{
     lines::{codepoint_len, substring_by_codepoint, LineNumber},
     positions::SingleLineSpan,
-    syntax::{MatchKind, MatchedPos},
+    syntax::{AtomKind, MatchKind, MatchedPos, TokenKind},
 };
 use colored::*;
 use std::{cmp::min, collections::HashMap};
@@ -13,6 +13,7 @@ struct Style {
     foreground: Color,
     background: Option<Color>,
     bold: bool,
+    dimmed: bool,
 }
 
 impl Style {
@@ -21,6 +22,9 @@ impl Style {
         if self.bold {
             res = res.bold();
         }
+        if self.dimmed {
+            res = res.dimmed();
+        }
         if let Some(background) = self.background {
             res = res.on_color(background);
         };
@@ -32,7 +36,7 @@ impl Style {
 /// Dim any parts of the line that have no spans.
 fn apply_line(line: &str, styles: &[(SingleLineSpan, Style)]) -> String {
     if styles.is_empty() {
-        return line.dimmed().to_string();
+        return line.purple().to_string();
     }
 
     let mut res = String::with_capacity(line.len());
@@ -98,11 +102,13 @@ fn apply(s: &str, styles: &[(SingleLineSpan, Style)]) -> String {
 pub fn apply_colors(s: &str, is_lhs: bool, positions: &[MatchedPos]) -> String {
     let mut styles = vec![];
     for pos in positions {
+        let line_pos = pos.pos;
         let style = match pos.kind {
-            MatchKind::Unchanged { .. } => Style {
+            MatchKind::Unchanged { highlight, .. } => Style {
                 foreground: Color::White,
                 background: None,
-                bold: false,
+                bold: highlight == TokenKind::Atom(AtomKind::Keyword),
+                dimmed: highlight == TokenKind::Atom(AtomKind::Comment),
             },
             MatchKind::Novel | MatchKind::ChangedCommentPart => Style {
                 foreground: if is_lhs {
@@ -112,16 +118,16 @@ pub fn apply_colors(s: &str, is_lhs: bool, positions: &[MatchedPos]) -> String {
                 },
                 background: None,
                 bold: true,
+                dimmed: false,
             },
             MatchKind::UnchangedCommentPart { .. } => Style {
                 foreground: if is_lhs { Color::Red } else { Color::Green },
                 background: None,
                 bold: false,
+                dimmed: false,
             },
         };
-        for line_pos in &pos.pos {
-            styles.push((*line_pos, style));
-        }
+        styles.push((line_pos, style));
     }
 
     apply(s, &styles)
