@@ -3,6 +3,8 @@
 #![allow(clippy::mutable_key_type)] // Hash for Syntax doesn't use mutable fields.
 
 use itertools::{EitherOrBoth, Itertools};
+use lazy_static::lazy_static;
+use regex::Regex;
 use std::{
     cell::Cell,
     cmp::min,
@@ -67,6 +69,12 @@ impl<'a> SyntaxInfo<'a> {
     }
 }
 
+impl<'a> Default for SyntaxInfo<'a> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 pub enum Syntax<'a> {
     List {
         info: SyntaxInfo<'a>,
@@ -81,7 +89,7 @@ pub enum Syntax<'a> {
         info: SyntaxInfo<'a>,
         position: Vec<SingleLineSpan>,
         content: String,
-        is_comment: bool,
+        kind: AtomKind,
     },
 }
 
@@ -137,6 +145,7 @@ impl<'a> fmt::Debug for Syntax<'a> {
                 content,
                 position,
                 info,
+                kind: highlight,
                 ..
             } => {
                 let mut ds = f.debug_struct(&format!(
@@ -148,6 +157,7 @@ impl<'a> fmt::Debug for Syntax<'a> {
                 ds.field("position", &dbg_pos(position));
 
                 if env::var("DFT_VERBOSE").is_ok() {
+                    ds.field("highlight", highlight);
                     ds.field("change", &info.change.get());
                     let next_s = match info.next.get() {
                         Some(List { .. }) => "Some(List)",
@@ -183,7 +193,7 @@ impl<'a> Syntax<'a> {
         }
 
         arena.alloc(List {
-            info: SyntaxInfo::new(),
+            info: SyntaxInfo::default(),
             open_position,
             open_content: open_content.into(),
             close_content: close_content.into(),
@@ -197,30 +207,13 @@ impl<'a> Syntax<'a> {
         arena: &'a Arena<Syntax<'a>>,
         position: Vec<SingleLineSpan>,
         content: &str,
+        kind: AtomKind,
     ) -> &'a Syntax<'a> {
-        Self::new_atom_(arena, position, content, false)
-    }
-
-    pub fn new_comment(
-        arena: &'a Arena<Syntax<'a>>,
-        position: Vec<SingleLineSpan>,
-        content: &str,
-    ) -> &'a Syntax<'a> {
-        Self::new_atom_(arena, position, content, true)
-    }
-
-    #[allow(clippy::mut_from_ref)] // Clippy doesn't understand arenas.
-    fn new_atom_(
-        arena: &'a Arena<Syntax<'a>>,
-        position: Vec<SingleLineSpan>,
-        content: &str,
-        is_comment: bool,
-    ) -> &'a mut Syntax<'a> {
         arena.alloc(Atom {
-            info: SyntaxInfo::new(),
+            info: SyntaxInfo::default(),
             position,
             content: content.into(),
-            is_comment,
+            kind,
         })
     }
 
@@ -313,7 +306,13 @@ pub fn init_info<'a>(lhs_roots: &[&'a Syntax<'a>], rhs_roots: &[&'a Syntax<'a>])
     set_content_id(rhs_roots, &mut existing);
 }
 
-type ContentKey = (Option<String>, Option<String>, Vec<u32>, bool, bool);
+type ContentKey = (
+    Option<String>,
+    Option<String>,
+    Vec<u32>,
+    bool,
+    Option<AtomKind>,
+);
 
 fn set_content_id<'a>(nodes: &[&'a Syntax<'a>], existing: &mut HashMap<ContentKey, u32>) {
     for node in nodes {
@@ -335,25 +334,26 @@ fn set_content_id<'a>(nodes: &[&'a Syntax<'a>], existing: &mut HashMap<ContentKe
                     Some(close_content.clone()),
                     children_content_ids,
                     true,
-                    false,
+                    None,
                 )
             }
             Atom {
                 content,
-                is_comment,
+                kind: highlight,
                 ..
             } => {
-                let clean_content = if *is_comment && content.lines().count() > 1 {
-                    content
-                        .lines()
-                        .map(|l| l.trim_start())
-                        .collect::<Vec<_>>()
-                        .join("\n")
-                        .to_string()
-                } else {
-                    content.clone()
-                };
-                (Some(clean_content), None, vec![], false, *is_comment)
+                let clean_content =
+                    if *highlight == AtomKind::Comment && content.lines().count() > 1 {
+                        content
+                            .lines()
+                            .map(|l| l.trim_start())
+                            .collect::<Vec<_>>()
+                            .join("\n")
+                            .to_string()
+                    } else {
+                        content.clone()
+                    };
+                (Some(clean_content), None, vec![], false, Some(*highlight))
             }
         };
 
@@ -476,9 +476,25 @@ impl<'a> Hash for Syntax<'a> {
     }
 }
 
+#[derive(PartialEq, Eq, Debug, Clone, Copy, Hash)]
+pub enum AtomKind {
+    Normal,
+    Comment,
+    Keyword,
+}
+
+/// Unlike atoms, tokens can be delimiters like `{`.
+#[derive(PartialEq, Eq, Debug, Clone, Copy)]
+pub enum TokenKind {
+    Delimiter,
+    Atom(AtomKind),
+}
+
 #[derive(PartialEq, Eq, Debug, Clone)]
 pub enum MatchKind {
     Unchanged {
+        highlight: TokenKind,
+        // as this match could be for a list.
         opposite_pos: (Vec<SingleLineSpan>, Vec<SingleLineSpan>),
     },
     Novel,
@@ -497,29 +513,31 @@ impl MatchKind {
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct MatchedPos {
     pub kind: MatchKind,
-    pub pos: Vec<SingleLineSpan>,
+    pub pos: SingleLineSpan,
     // TODO: this is confusing: the previous syntax node with a match
     // may be on the current line or a previous one.
-    pub prev_opposite_pos: Vec<SingleLineSpan>,
+    pub prev_opposite_pos: Option<SingleLineSpan>,
+}
+
+fn split_words(s: &str) -> Vec<String> {
+    lazy_static! {
+        static ref RE: Regex = Regex::new(r"[a-zA-Z0-9]+|[^a-zA-Z0-9]+").unwrap();
+    }
+
+    RE.find_iter(s).map(|m| m.as_str().to_owned()).collect()
 }
 
 fn split_comment_words(
     content: &str,
-    pos: &[SingleLineSpan],
+    pos: SingleLineSpan,
     opposite_content: &str,
-    opposite_pos: &[SingleLineSpan],
+    opposite_pos: SingleLineSpan,
     prev_opposite_pos: &[SingleLineSpan],
 ) -> Vec<MatchedPos> {
-    // TODO: also split on whitespace, so "// (foo)" splits before "(".
-
     // TODO: merge adjacent single-line comments unless there are
     // blank lines between them.
-    let content_parts: Vec<_> = content
-        .split_inclusive(&[' ', '\n', '\t'] as &[char])
-        .collect();
-    let other_parts: Vec<_> = opposite_content
-        .split_inclusive(&[' ', '\n', '\t'] as &[char])
-        .collect();
+    let content_parts = split_words(content);
+    let other_parts = split_words(opposite_content);
 
     let content_newlines = NewlinePositions::from(content);
     let opposite_content_newlines = NewlinePositions::from(opposite_content);
@@ -535,20 +553,20 @@ fn split_comment_words(
                 res.push(MatchedPos {
                     kind: MatchKind::ChangedCommentPart,
                     pos: content_newlines.from_offsets_relative_to(
-                        pos[0],
+                        pos,
                         offset,
                         offset + word.len(),
-                    ),
-                    prev_opposite_pos: prev_opposite_pos.to_vec(),
+                    )[0],
+                    prev_opposite_pos: prev_opposite_pos.first().copied(),
                 });
                 offset += word.len();
             }
             diff::Result::Both(word, opposite_word) => {
                 // This word is present on both sides.
                 let word_pos =
-                    content_newlines.from_offsets_relative_to(pos[0], offset, offset + word.len());
+                    content_newlines.from_offsets_relative_to(pos, offset, offset + word.len())[0];
                 let opposite_word_pos = opposite_content_newlines.from_offsets_relative_to(
-                    opposite_pos[0],
+                    opposite_pos,
                     opposite_offset,
                     opposite_offset + opposite_word.len(),
                 );
@@ -558,7 +576,7 @@ fn split_comment_words(
                         opposite_pos: opposite_word_pos,
                     },
                     pos: word_pos,
-                    prev_opposite_pos: prev_opposite_pos.to_vec(),
+                    prev_opposite_pos: prev_opposite_pos.first().copied(),
                 });
                 offset += word.len();
                 opposite_offset += opposite_word.len();
@@ -576,8 +594,9 @@ fn split_comment_words(
 impl MatchedPos {
     fn new(
         ck: ChangeKind,
-        pos: Vec<SingleLineSpan>,
-        prev_opposite_pos: Vec<SingleLineSpan>,
+        highlight: TokenKind,
+        pos: &[SingleLineSpan],
+        prev_opposite_pos: &[SingleLineSpan],
     ) -> Vec<Self> {
         let kind = match ck {
             ReplacedComment(this, opposite) => {
@@ -594,10 +613,11 @@ impl MatchedPos {
 
                 return split_comment_words(
                     this_content,
-                    &pos,
+                    // TODO: handle the whole pos here.
+                    pos[0],
                     opposite_content,
-                    opposite_pos,
-                    &prev_opposite_pos,
+                    opposite_pos[0],
+                    prev_opposite_pos,
                 );
             }
             Unchanged(opposite) => {
@@ -610,16 +630,33 @@ impl MatchedPos {
                     Atom { position, .. } => (position.clone(), position.clone()),
                 };
 
-                MatchKind::Unchanged { opposite_pos }
+                MatchKind::Unchanged {
+                    highlight,
+                    opposite_pos,
+                }
             }
             Novel => MatchKind::Novel,
         };
 
-        vec![Self {
-            kind,
-            pos,
-            prev_opposite_pos,
-        }]
+        // Create a MatchedPos for every line that `pos` covers.
+        let mut res = vec![];
+        for (i, line_pos) in pos.iter().enumerate() {
+            // Try to take line N on the opposite side for line N in
+            // this position. If the opposite position is fewer lines,
+            // just take the last line.
+            let opposite_line_pos = prev_opposite_pos
+                .get(i)
+                .or_else(|| prev_opposite_pos.last())
+                .copied();
+
+            res.push(Self {
+                kind: kind.clone(),
+                pos: *line_pos,
+                prev_opposite_pos: opposite_line_pos,
+            });
+        }
+
+        res
     }
 }
 
@@ -683,8 +720,9 @@ fn change_positions_<'a>(
 
                 positions.extend(MatchedPos::new(
                     change,
-                    open_position.clone(),
-                    prev_opposite_pos.clone(),
+                    TokenKind::Delimiter,
+                    open_position,
+                    prev_opposite_pos,
                 ));
 
                 change_positions_(
@@ -708,11 +746,17 @@ fn change_positions_<'a>(
                 }
                 positions.extend(MatchedPos::new(
                     change,
-                    close_position.clone(),
-                    prev_opposite_pos.clone(),
+                    TokenKind::Delimiter,
+                    close_position,
+                    prev_opposite_pos,
                 ));
             }
-            Atom { info, position, .. } => {
+            Atom {
+                info,
+                position,
+                kind,
+                ..
+            } => {
                 let change = info
                     .change
                     .get()
@@ -733,8 +777,9 @@ fn change_positions_<'a>(
                 }
                 positions.extend(MatchedPos::new(
                     change,
-                    position.clone(),
-                    prev_opposite_pos.clone(),
+                    TokenKind::Atom(*kind),
+                    position,
+                    prev_opposite_pos,
                 ));
             }
         }
@@ -1053,6 +1098,7 @@ mod tests {
                 end_col: 3,
             }],
             "foo",
+            AtomKind::Normal,
         );
         atom.set_change(ChangeKind::Novel);
         let nodes: Vec<&Syntax> = vec![atom];
@@ -1060,11 +1106,11 @@ mod tests {
         let positions = change_positions("irrelevant", "also irrelevant", &nodes);
         assert_eq!(
             positions[0].prev_opposite_pos,
-            vec![SingleLineSpan {
+            Some(SingleLineSpan {
                 line: 0.into(),
                 start_col: 0,
                 end_col: 0
-            }]
+            })
         );
     }
 
@@ -1078,8 +1124,8 @@ mod tests {
 
         let arena = Arena::new();
 
-        let comment = Syntax::new_comment(&arena, pos.clone(), "foo");
-        let atom = Syntax::new_atom(&arena, pos, "foo");
+        let comment = Syntax::new_atom(&arena, pos.clone(), "foo", AtomKind::Comment);
+        let atom = Syntax::new_atom(&arena, pos, "foo", AtomKind::Normal);
         init_info(&[comment], &[atom]);
 
         assert_ne!(comment, atom);
@@ -1095,8 +1141,8 @@ mod tests {
 
         let arena = Arena::new();
 
-        let x = Syntax::new_comment(&arena, pos.clone(), "foo\nbar");
-        let y = Syntax::new_comment(&arena, pos, "foo\n    bar");
+        let x = Syntax::new_atom(&arena, pos.clone(), "foo\nbar", AtomKind::Comment);
+        let y = Syntax::new_atom(&arena, pos, "foo\n    bar", AtomKind::Comment);
         init_info(&[x], &[y]);
 
         assert_eq!(x, y);
@@ -1107,7 +1153,7 @@ mod tests {
         let lhs = Atom {
             info: SyntaxInfo {
                 change: Cell::new(Some(Novel)),
-                ..SyntaxInfo::new()
+                ..SyntaxInfo::default()
             },
 
             position: vec![SingleLineSpan {
@@ -1116,12 +1162,12 @@ mod tests {
                 end_col: 3,
             }],
             content: "foo".into(),
-            is_comment: false,
+            kind: AtomKind::Normal,
         };
         let rhs = Atom {
             info: SyntaxInfo {
                 change: Cell::new(None),
-                ..SyntaxInfo::new()
+                ..SyntaxInfo::default()
             },
             position: vec![SingleLineSpan {
                 line: 1.into(),
@@ -1129,7 +1175,7 @@ mod tests {
                 end_col: 3,
             }],
             content: "foo".into(),
-            is_comment: false,
+            kind: AtomKind::Normal,
         };
         init_info(&[&lhs], &[&rhs]);
 
@@ -1139,31 +1185,38 @@ mod tests {
     #[test]
     fn test_split_comment_words_basic() {
         let content = "abc";
-        let pos = vec![SingleLineSpan {
+        let pos = SingleLineSpan {
             line: 0.into(),
             start_col: 0,
             end_col: 3,
-        }];
+        };
 
         let opposite_content = "def";
-        let opposite_pos = vec![SingleLineSpan {
+        let opposite_pos = SingleLineSpan {
             line: 0.into(),
             start_col: 0,
             end_col: 3,
-        }];
+        };
 
-        let res = split_comment_words(content, &pos, opposite_content, &opposite_pos, &[]);
+        let res = split_comment_words(content, pos, opposite_content, opposite_pos, &[]);
         assert_eq!(
             res,
             vec![MatchedPos {
                 kind: MatchKind::ChangedCommentPart,
-                pos: vec![SingleLineSpan {
+                pos: SingleLineSpan {
                     line: 0.into(),
                     start_col: 0,
                     end_col: 3
-                }],
-                prev_opposite_pos: vec![]
+                },
+                prev_opposite_pos: None,
             },]
         );
     }
+
+    #[test]
+    fn test_split_words() {
+        let s = "example.com";
+        let res = split_words(s);
+        assert_eq!(res, vec!["example", ".", "com"])
+    }
 }
